@@ -1,8 +1,7 @@
 /*****************************************************************************
  * aggregartp.c: split an RTP stream for several contribution links
  *****************************************************************************
- * Copyright (C) 2009, 2011, 2014-2015 VideoLAN
- * $Id$
+ * Copyright (C) 2009, 2011, 2014-2017 VideoLAN
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -87,6 +86,7 @@ static void usage(void)
 {
     msg_Raw( NULL, "Usage: aggregartp [-i <RT priority>] [-l <syslogtag>] [-t <ttl>] [-w] [-o <SSRC IP>] [-U] [-x <retx buffer>] [-X <retx URL>] [-m <payload size>] [-R <RTP header>] @<src host> <dest host 1>[,<weight 1>] ... [<dest host N>,<weight N>]" );
     msg_Raw( NULL, "    host format: [<connect addr>[:<connect port>]][@[<bind addr][:<bind port>]]" );
+    msg_Raw( NULL, "    weight: integer, higher value means more capacity, or 0 for all packets" );
     msg_Raw( NULL, "    -w: overwrite RTP timestamps" );
     msg_Raw( NULL, "    -o: overwrite RTP SSRC" );
     msg_Raw( NULL, "    -U: prepend RTP header" );
@@ -140,6 +140,17 @@ static void SendBlock( int i_fd, struct sockaddr *p_sout,
     }
 }
 
+/*****************************************************************************
+ * SendBlock0: send a block to all outputs with weight 0
+ *****************************************************************************/
+static void SendBlock0( block_t *p_block )
+{
+    int i;
+    for ( i = 0; i < i_nb_outputs; i++ )
+        if ( !p_outputs[i].i_weight )
+            SendBlock( p_outputs[i].i_fd, NULL, 0, p_block);
+}
+
 /*****************************************************************************
  * RetxQueue: store a packet in the retx queue
  *****************************************************************************/
@@ -345,10 +356,15 @@ int main( int i_argc, char **pp_argv )
 
     while ( optind < i_argc )
     {
+        bool b_multicast;
+        struct opensocket_opt opt;
+        memset(&opt, 0, sizeof(struct opensocket_opt));
+        opt.pb_multicast = &b_multicast;
+
         p_outputs = realloc( p_outputs, ++i_nb_outputs * sizeof(output_t) );
         p_outputs[i_nb_outputs - 1].i_fd = i_fd =
             OpenSocket( pp_argv[optind++], i_ttl, 0, DEFAULT_PORT,
-                        &p_outputs[i_nb_outputs - 1].i_weight, NULL, NULL );
+                        &p_outputs[i_nb_outputs - 1].i_weight, NULL, &opt );
         if ( p_outputs[i_nb_outputs - 1].i_fd == -1 )
         {
             msg_Err( NULL, "unable to open output socket" );
@@ -359,13 +375,13 @@ int main( int i_argc, char **pp_argv )
             p_outputs[i_nb_outputs - 1].i_remainder = 0;
         i_max_weight += p_outputs[i_nb_outputs - 1].i_weight;
 
-        if ( i_retx_fd == -1 )
+        if ( i_retx_fd == -1 && !b_multicast )
         {
             ADD_RETX
         }
     }
     msg_Dbg( NULL, "%d outputs weight %u%s", i_nb_outputs, i_max_weight,
-             i_retx_fd != -1 ? ", with retx" : "" );
+             i_nb_retx > 1 ? ", with retx" : "" );
 
     p_retx_block = malloc( sizeof(block_t) + RETX_HEADER_SIZE );
     p_retx_block->p_data = (uint8_t *)p_retx_block + sizeof(block_t);
@@ -389,7 +405,7 @@ int main( int i_argc, char **pp_argv )
     for ( ; ; )
     {
         uint64_t i_current_date;
-        if ( poll( pfd, i_nb_retx + 1, -1 ) < 0 )
+        if ( poll( pfd, i_nb_retx, -1 ) < 0 )
         {
             int saved_errno = errno;
             msg_Warn( NULL, "couldn't poll(): %s", strerror(errno) );
@@ -472,13 +488,18 @@ int main( int i_argc, char **pp_argv )
             }
 
             /* Output block */
-            output_t *p_output = NextOutput();
-            SendBlock( p_output->i_fd, NULL, 0, p_input_block );
+            SendBlock0( p_input_block );
 
-            p_output->i_weighted_size += (i_size + p_output->i_remainder)
-                                           / p_output->i_weight;
-            p_output->i_remainder = (i_size + p_output->i_remainder)
-                                           % p_output->i_weight;
+            if ( i_max_weight )
+            {
+                output_t *p_output = NextOutput();
+                SendBlock( p_output->i_fd, NULL, 0, p_input_block );
+
+                p_output->i_weighted_size += (i_size + p_output->i_remainder)
+                                               / p_output->i_weight;
+                p_output->i_remainder = (i_size + p_output->i_remainder)
+                                               % p_output->i_weight;
+            }
 
             RetxQueue( p_input_block, i_current_date );
             p_input_block = NULL;
@@ -490,11 +511,11 @@ int main( int i_argc, char **pp_argv )
         }
 
         int i;
-        for ( i = 0; i < i_nb_retx; i++ )
+        for ( i = 1; i < i_nb_retx; i++ )
         {
-            if ( pfd[i + 1].revents & POLLIN )
+            if ( pfd[i].revents & POLLIN )
             {
-                RetxHandle( pfd[i + 1].fd );
+                RetxHandle( pfd[i].fd );
             }
         }
     }
