@@ -537,9 +537,9 @@ static char *config_stropt( char *psz_string )
     return ret;
 }
 
-int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
-                uint16_t i_connect_port, unsigned int *pi_weight, bool *pb_tcp,
-                struct opensocket_opt *p_opt)
+int OpenSocketSafe( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
+                    uint16_t i_connect_port, unsigned int *pi_weight, bool *pb_tcp,
+                    struct opensocket_opt *p_opt)
 {
     sockaddr_t bind_addr, connect_addr;
     int i_fd = -1, i;
@@ -569,8 +569,15 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
         pb_tcp = &b_tcp;
     *pb_tcp = false;
 
-    if ( p_opt != NULL && p_opt->pb_multicast != NULL )
-        *p_opt->pb_multicast = false;
+    if ( p_opt )
+    {
+        if ( p_opt->pb_multicast )
+            *p_opt->pb_multicast = false;
+        if ( p_opt->pb_raw_packets )
+            *p_opt->pb_raw_packets = false;
+        if ( p_opt->pb_udp )
+            *p_opt->pb_udp = false;
+    }
 
     psz_token2 = strrchr( psz_arg, ',' );
     if ( psz_token2 )
@@ -659,6 +666,11 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
                 i_tos = strtol( ARG_OPTION("tos="), NULL, 0 );
             else if ( IS_OPTION("tcp") )
                 *pb_tcp = true;
+            else if ( IS_OPTION("udp") )
+            {
+                if ( p_opt && p_opt->pb_udp )
+                    *p_opt->pb_udp = true;
+            }
             else if ( IS_OPTION("srcaddr=") )
             {
                 char *option = config_stropt( ARG_OPTION("srcaddr=") );
@@ -686,7 +698,7 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
           && bind_addr.ss.ss_family != connect_addr.ss.ss_family )
     {
         msg_Err( NULL, "incompatible address types" );
-        exit(EXIT_FAILURE);
+        return -2;
     }
     if ( bind_addr.ss.ss_family != AF_UNSPEC )
         i_family = bind_addr.ss.ss_family;
@@ -695,7 +707,7 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
     else
     {
         msg_Err( NULL, "ambiguous address declaration" );
-        exit(EXIT_FAILURE);
+        return -2;
     }
     i_sockaddr_len = (i_family == AF_INET) ? sizeof(struct sockaddr_in) :
                      sizeof(struct sockaddr_in6);
@@ -704,7 +716,7 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
           && i_bind_if_index != i_connect_if_index )
     {
         msg_Err( NULL, "incompatible bind and connect interfaces" );
-        exit(EXIT_FAILURE);
+        return -2;
     }
     if ( i_connect_if_index ) i_bind_if_index = i_connect_if_index;
     else i_connect_if_index = i_bind_if_index;
@@ -712,17 +724,20 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
     /* Socket configuration */
     if ( i_fd < 0 )
     {
-        if (b_raw_packets && b_host)
-        { 
+        if (b_raw_packets && i_raw_srcaddr != INADDR_ANY && b_host)
+        {
             RawFillHeaders(p_opt->p_raw_pktheader,
                 i_raw_srcaddr, connect_addr.sin.sin_addr.s_addr, i_raw_srcport,
                 ntohs(connect_addr.sin.sin_port), i_ttl, i_tos, 0);
             i_fd = socket( AF_INET, SOCK_RAW, IPPROTO_RAW );
+            if ( p_opt->pb_raw_packets != NULL )
+                *p_opt->pb_raw_packets = true;
 #ifdef __FreeBSD__
             if ( setsockopt( i_fd, IPPROTO_IP, IP_HDRINCL, &hincl, sizeof(hincl)) == -1 )
             {
                 msg_Err( NULL, "unable to set socket (%s)", strerror(errno) );
-                exit(EXIT_FAILURE);
+                close(i_fd);
+                return -2;
             }
 #endif
         } else
@@ -731,7 +746,8 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
         if ( i_fd < 0 )
         {
             msg_Err( NULL, "unable to open socket (%s)", strerror(errno) );
-            exit(EXIT_FAILURE);
+            close(i_fd);
+            return -2;
         }
 
         i = 1;
@@ -739,7 +755,8 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
                          sizeof(i) ) == -1 )
         {
             msg_Err( NULL, "unable to set socket (%s)", strerror(errno) );
-            exit(EXIT_FAILURE);
+            close(i_fd);
+            return -2;
         }
 
         if ( i_family == AF_INET6 )
@@ -750,7 +767,8 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
             {
                 msg_Err( NULL, "couldn't set interface index" );
                 PrintSocket( "socket definition:", &bind_addr, &connect_addr );
-                exit(EXIT_FAILURE);
+                close(i_fd);
+                return -2;
             }
 
             if ( bind_addr.ss.ss_family != AF_UNSPEC )
@@ -768,7 +786,8 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
                         msg_Err( NULL, "couldn't bind" );
                         PrintSocket( "socket definition:", &bind_addr,
                                      &connect_addr );
-                        exit(EXIT_FAILURE);
+                        close(i_fd);
+                        return -2;
                     }
 
                     imr.ipv6mr_multiaddr = bind_addr.sin6.sin6_addr;
@@ -781,7 +800,8 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
                         msg_Err( NULL, "couldn't join multicast group" );
                         PrintSocket( "socket definition:", &bind_addr,
                                       &connect_addr );
-                        exit(EXIT_FAILURE);
+                        close( i_fd );
+                        return -2;
                     }
 
                     if ( p_opt != NULL && p_opt->pb_multicast != NULL )
@@ -799,7 +819,8 @@ normal_bind:
             {
                 msg_Err( NULL, "couldn't bind" );
                 PrintSocket( "socket definition:", &bind_addr, &connect_addr );
-                exit(EXIT_FAILURE);
+                close( i_fd );
+                return -2;
             }
         }
     }
@@ -833,7 +854,8 @@ normal_bind:
                              strerror(errno) );
                     PrintSocket( "socket definition:", &bind_addr,
                                  &connect_addr );
-                    exit(EXIT_FAILURE);
+                    close( i_fd );
+                    return -2;
                 }
             }
             else
@@ -854,7 +876,8 @@ normal_bind:
                              strerror(errno) );
                     PrintSocket( "socket definition:", &bind_addr,
                                  &connect_addr );
-                    exit(EXIT_FAILURE);
+                    close( i_fd );
+                    return -2;
                 }
             }
             else
@@ -872,7 +895,8 @@ normal_bind:
                              strerror(errno) );
                     PrintSocket( "socket definition:", &bind_addr,
                                  &connect_addr );
-                    exit(EXIT_FAILURE);
+                    close( i_fd );
+                    return -2;
                 }
             }
 #ifdef SO_BINDTODEVICE
@@ -881,7 +905,8 @@ normal_bind:
                                  psz_ifname, strlen(psz_ifname)+1 ) < 0 ) {
                     msg_Err( NULL, "couldn't bind to device %s (%s)",
                              psz_ifname, strerror(errno) );
-                    exit(EXIT_FAILURE);
+                    close( i_fd );
+                    return -2;
                 }
                 free(psz_ifname);
                 psz_ifname = NULL;
@@ -900,39 +925,47 @@ normal_bind:
             msg_Err( NULL, "cannot connect socket (%s)",
                      strerror(errno) );
             PrintSocket( "socket definition:", &bind_addr, &connect_addr );
-            exit(EXIT_FAILURE);
+            close( i_fd );
+            return -2;
         }
 
         if ( !*pb_tcp )
         {
-            if ( i_ttl )
+            if ( i_family == AF_INET
+                  && IN_MULTICAST(ntohl(connect_addr.sin.sin_addr.s_addr)) )
             {
-                if ( i_family == AF_INET
-                      && IN_MULTICAST(ntohl(connect_addr.sin.sin_addr.s_addr)) )
-                {
-                    if ( setsockopt( i_fd, IPPROTO_IP, IP_MULTICAST_TTL,
-                                     (void *)&i_ttl, sizeof(i_ttl) ) == -1 )
-                    {
-                        msg_Err( NULL, "couldn't set TTL (%s)",
-                                 strerror(errno) );
-                        PrintSocket( "socket definition:", &bind_addr,
-                                     &connect_addr );
-                        exit(EXIT_FAILURE);
-                    }
-                }
+                if ( p_opt != NULL && p_opt->pb_multicast != NULL )
+                    *p_opt->pb_multicast = true;
 
-                if ( i_family == AF_INET6
-                      && IN6_IS_ADDR_MULTICAST(&connect_addr.sin6.sin6_addr) )
+                if ( i_ttl &&
+                     setsockopt( i_fd, IPPROTO_IP, IP_MULTICAST_TTL,
+                                 (void *)&i_ttl, sizeof(i_ttl) ) == -1 )
                 {
-                    if ( setsockopt( i_fd, IPPROTO_IPV6, IPV6_MULTICAST_HOPS,
-                                     (void *)&i_ttl, sizeof(i_ttl) ) == -1 )
-                    {
-                        msg_Err( NULL, "couldn't set TTL (%s)",
-                                 strerror(errno) );
-                        PrintSocket( "socket definition:", &bind_addr,
-                                     &connect_addr );
-                        exit(EXIT_FAILURE);
-                    }
+                    msg_Err( NULL, "couldn't set TTL (%s)",
+                             strerror(errno) );
+                    PrintSocket( "socket definition:", &bind_addr,
+                                 &connect_addr );
+                    close( i_fd );
+                    return -2;
+                }
+            }
+
+            if ( i_family == AF_INET6
+                  && IN6_IS_ADDR_MULTICAST(&connect_addr.sin6.sin6_addr) )
+            {
+                if ( p_opt != NULL && p_opt->pb_multicast != NULL )
+                    *p_opt->pb_multicast = true;
+
+                if ( i_ttl &&
+                     setsockopt( i_fd, IPPROTO_IPV6, IPV6_MULTICAST_HOPS,
+                                 (void *)&i_ttl, sizeof(i_ttl) ) == -1 )
+                {
+                    msg_Err( NULL, "couldn't set TTL (%s)",
+                             strerror(errno) );
+                    PrintSocket( "socket definition:", &bind_addr,
+                                 &connect_addr );
+                    close( i_fd );
+                    return -2;
                 }
             }
 
@@ -944,7 +977,8 @@ normal_bind:
                     msg_Err( NULL, "couldn't set TOS (%s)", strerror(errno) );
                     PrintSocket( "socket definition:", &bind_addr,
                                  &connect_addr );
-                    exit(EXIT_FAILURE);
+                    close( i_fd );
+                    return -2;
                 }
             }
         }
@@ -957,7 +991,8 @@ normal_bind:
         {
             msg_Err( NULL, "couldn't listen (%s)", strerror(errno) );
             PrintSocket( "socket definition:", &bind_addr, &connect_addr );
-            exit(EXIT_FAILURE);
+            close( i_fd );
+            return -2;
         }
 
         while ( (i_new_fd = accept( i_fd, NULL, NULL )) < 0 )
@@ -966,7 +1001,8 @@ normal_bind:
             {
                 msg_Err( NULL, "couldn't accept (%s)", strerror(errno) );
                 PrintSocket( "socket definition:", &bind_addr, &connect_addr );
-                exit(EXIT_FAILURE);
+                close( i_fd );
+                return -2;
             }
         }
         close( i_fd );
@@ -976,6 +1012,18 @@ normal_bind:
     return i_fd;
 }
 
+int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
+                uint16_t i_connect_port, unsigned int *pi_weight, bool *pb_tcp,
+                struct opensocket_opt *p_opt)
+{
+    int ret = OpenSocketSafe(_psz_arg, i_ttl, i_bind_port,
+                             i_connect_port, pi_weight, pb_tcp, p_opt);
+    if (ret == -2)
+        exit(EXIT_FAILURE);
+
+    return ret;
+}
+
 /*****************************************************************************
  * StatFile: parse argv and return the mode of the named file
  *****************************************************************************/
