@@ -40,6 +40,7 @@
 #define DEFAULT_ROTATE_OFFSET UINT64_C(0)
 #define TS_SIZE 188
 #define RTP_HEADER_SIZE 12
+#define CLOCK_FREQ UINT64_C(27000000)
 
 #define VERB_DBG  3
 #define VERB_INFO 2
@@ -76,6 +77,8 @@ struct udprawpkt {
  struct opensocket_opt {
     struct udprawpkt *p_raw_pktheader;
     bool *pb_multicast;
+    bool *pb_raw_packets;
+    bool *pb_udp;
  };
 
 
@@ -93,6 +96,9 @@ uint64_t wall_Date( void );
 void wall_Sleep( uint64_t i_delay );
 uint64_t real_Date( void );
 void real_Sleep( uint64_t i_delay );
+int OpenSocketSafe( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
+                    uint16_t i_connect_port, unsigned int *pi_weight, bool *pb_tcp,
+                    struct opensocket_opt *p_opt);
 int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
                 uint16_t i_connect_port, unsigned int *pi_weight, bool *pb_tcp,
                 struct opensocket_opt *p_opt);
